==> 1.1.1.tar.gz: lib/src/entities/user.ent.dart <==
import 'package:collection/collection.dart' show IterableExtension;
import 'package:rec_api_dart/src/entities/account.ent.dart';
import 'package:rec_api_dart/src/entities/campaign.ent.dart';
import 'package:rec_api_dart/src/entities/entity.base.dart';
import 'package:rec_api_dart/src/entities/level.ent.dart';
import 'package:rec_api_dart/src/entities/role.ent.dart';
import 'package:rec_api_dart/src/helpers/checks.dart';

class User extends Entity {
  List<Account> accounts = [];
  List<Campaign> campaigns = [];
  List<Role> roles = [];

  Account? selectedAccount;
  String? username;
  String? image;
  String phone;
  String prefix;
  String? email;

  // Info required for Culture campaign
  num? kycId;
  num? zip;
  String? birthDate;
  String? gender;

  bool? hasPin = false;

  /// Whether the user has already accepted the terms of service of the LTAB20 campaign.
  bool? privateTosLtab = false;

  /// Whether the user has already accepted the terms of service of the CULT21 campaign.
  bool? privateTosCulture = false;

  bool get hasExtraData {
    return zip != null &&
        (birthDate != null && birthDate!.isNotEmpty) &&
        (gender != null && gender!.isNotEmpty);
  }

  User({
    String? id,
    String? createdAt,
    String? updatedAt,
    this.username,
    this.accounts = const [],
    this.roles = const [],
    this.selectedAccount,
    this.hasPin = false,
    this.privateTosLtab = false,
    this.privateTosCulture = false,
    this.phone = '',
    this.prefix = '',
    this.email = '',
    this.zip,
    this.birthDate = '',
    this.gender = '',
    this.kycId,
  }) : super(id, createdAt, updatedAt);

  bool isCampaignActive(Campaign campaign) {
    return campaigns.contains(campaign);
  }

  bool isCampaignActiveById(String campaignId) {
    // ignore: unnecessary_null_comparison
    return campaigns.where((el) => el.id == campaignId) != null;
  }

  bool hasCampaignAccount(String? campaignCode) {
    return getAccountForCampaign(campaignCode) != null;
  }

  /// Returns the campaign account for this user
  Account? getCampaignAccount(String campaignCode) {
    if (Checks.isEmpty(accounts)) return null;

    var ltabAccounts = accounts.where((el) => el.isPrivate()).where((account) {
      if (account.campaigns.isEmpty) return false;

      var matchingCampaign =
          account.campaigns.where((campaign) => campaign.id == campaignCode);

      // ignore: unnecessary_null_comparison
      return matchingCampaign != null && matchingCampaign.isNotEmpty;
    });

    return ltabAccounts.isNotEmpty ? ltabAccounts.last : null;
  }

  /// Returns the campaign account for this user
  Account? getAccountForCampaign(String? campaignCode) {
    if (Checks.isEmpty(accounts)) return null;

    return accounts.firstWhereOrNull(
      (account) => account.isCampaignAccount(campaignCode),
    );
  }

  /// Checks whether any [Account] of this [User] satisfies [test].
  /// Checks every account, and returns true if any of them make [test] return true, otherwise it returns false.
  bool anyAccountMatches(bool Function(Account account) test) {
    if (Checks.isEmpty(accounts)) return false;

    return accounts.any(test);
  }

  /// Checks whether this user has any account with a level that matches [code]
  /// Codes are available as constants in Level [Level.CODE_KYC2]
  bool anyAccountAtLevel(String code) {
    return anyAccountMatches(
      (account) => account.hasLevelByCode(code),
    );
  }

  /// Checks whether this user has any account with a level that matches [KYC2]
  bool anyAccountAtKyc2() {
    return anyAccountAtLevel(Level.CODE_KYC2);
  }

  String get formattedPhone => '(+$prefix) $phone';

  /// Checks whether this user has specified [validRoles] over the active account
  /// if [validRoles] is empty we asume that eveyone has the "NO Role" role.
  bool hasRoles(List<Role> validRoles) {
    if (validRoles.isEmpty) return true;

    for (var role in roles) {
      if (validRoles.contains(role)) {
        return true;
      }
    }

    return false;
  }

  factory User.fromJson(Map<String, dynamic> json) {
    var roles = <Role>[];

    if (json['roles'] != null) {
      var jsonRoles = List.from(json['roles']);
      roles = jsonRoles.isNotEmpty
          ? jsonRoles.map<Role>((el) => Role.getRoleByName(el)).toList()
          : <Role>[];
    }

    var kyc = json['kyc_validations'];

    return User(
      id: '${json['id']}',
      createdAt: json['created'],
      updatedAt: json['updated'],
      username: json['username'],
      phone: json['phone'].toString(),
      prefix: json['prefix'].toString(),
      hasPin: json['has_pin'],
      email: json['email'],
      kycId: kyc['id'],
      zip: kyc['zip'],
      birthDate: kyc['date_birth'],
      gender: kyc['gender'],
      privateTosLtab: json['private_tos_campaign'],
      privateTosCulture: json['private_tos_campaign_culture'],
      selectedAccount: Account.fromJson(json['active_group']),
      accounts: List.from(json['activeAccounts'])
          .map((el) => Account.fromJson(el))
          .toList(),
      roles: roles,
    );
  }
}

==> 1.4.3.tar.gz: lib/src/entities/user.ent.dart <==
import 'package:collection/collection.dart' show IterableExtension;
import '../helpers/checks.dart';
import 'account.ent.dart';
import 'campaign.ent.dart';
import 'entity.base.dart';
import 'level.ent.dart';
import 'role.ent.dart';
import 'bank_card.ent.dart';

class User extends Entity {
  List<Account> accounts = [];
  List<Campaign> campaigns = [];
  List<Role> roles = [];
  List<BankCard> bankCards = [];

  Account? selectedAccount;
  String? username;
  String? image;
  String phone;
  String prefix;
  String? email;

  // Info required for Culture campaign
  num? kycId;
  num? zip;
  String? birthDate;
  String? gender;

  bool? hasPin = false;
  bool hasSavedCards;

  /// Whether the user is the KYC Manager of the current selected account [selectedAccount]
  bool isKycManager = false;

  /// Whether the user has already accepted the terms of service of the LTAB20 campaign.
  bool? privateTosLtab = false;

  /// Whether the user has already accepted the terms of service of the CULT21 campaign.
  bool? privateTosCulture = false;

  bool get hasExtraData {
    return zip != null &&
        (birthDate != null && birthDate!.isNotEmpty) &&
        (gender != null && gender!.isNotEmpty);
  }

  User({
    String? id,
    String? createdAt,
    String? updatedAt,
    this.username,
    this.accounts = const [],
    this.roles = const [],
    this.bankCards = const [],
    this.selectedAccount,
    this.hasPin = false,
    this.privateTosLtab = false,
    this.privateTosCulture = false,
    this.hasSavedCards = false,
    this.isKycManager = false,
    this.phone = '',
    this.prefix = '',
    this.email = '',
    this.zip,
    this.birthDate = '',
    this.gender = '',
    this.kycId,
  }) : super(id, createdAt, updatedAt);

  bool isCampaignActive(Campaign campaign) {
    return campaigns.contains(campaign);
  }

  bool isCampaignActiveById(String campaignId) {
    // ignore: unnecessary_null_comparison
    return campaigns.where((el) => el.id == campaignId) != null;
  }

  bool hasCampaignAccount(String? campaignCode) {
    return getAccountForCampaign(campaignCode) != null;
  }

  /// Returns the campaign account for this user
  Account? getCampaignAccount(String campaignCode) {
    if (Checks.isEmpty(accounts)) return null;

    var ltabAccounts = accounts.where((el) => el.isPrivate()).where((account) {
      if (account.campaigns.isEmpty) return false;

      var matchingCampaign = account.campaigns.where((campaign) => campaign.id == campaignCode);

      // ignore: unnecessary_null_comparison
      return matchingCampaign != null && matchingCampaign.isNotEmpty;
    });

    return ltabAccounts.isNotEmpty ? ltabAccounts.last : null;
  }

  /// Returns the campaign account for this user
  Account? getAccountForCampaign(String? campaignCode) {
    if (Checks.isEmpty(accounts)) return null;

    return accounts.firstWhereOrNull(
      (account) => account.isCampaignAccount(campaignCode),
    );
  }

  /// Checks whether any [Account] of this [User] satisfies [test].
  /// Checks every account, and returns true if any of them make [test] return true, otherwise it returns false.
  bool anyAccountMatches(bool Function(Account account) test) {
    if (Checks.isEmpty(accounts)) return false;

    return accounts.any(test);
  }

  /// Checks whether this user has any account with a level that matches [code]
  /// Codes are available as constants in Level [Level.CODE_KYC2]
  bool anyAccountAtLevel(String code) {
    return anyAccountMatches(
      (account) => account.hasLevelByCode(code),
    );
  }

  /// Checks whether this user has any account with a level that matches [KYC2]
  bool anyAccountAtKyc2() {
    return anyAccountAtLevel(Level.CODE_KYC2);
  }

  String get formattedPhone => '(+$prefix) $phone';

  /// Checks whether this user has specified [validRoles] over the active account
  /// if [validRoles] is empty we asume that eveyone has the "NO Role" role.
  bool hasRoles(List<Role> validRoles) {
    if (validRoles.isEmpty) return true;

    for (var role in roles) {
      if (validRoles.contains(role)) {
        return true;
      }
    }

    return false;
  }

  factory User.fromJson(Map<String, dynamic> json) {
    var roles = <Role>[];
    var bankCards = <BankCard>[];

    if (json['roles'] != null) {
      final jsonRoles = List.from(json['roles']);
      roles = jsonRoles.isNotEmpty
          ? jsonRoles.map<Role>((el) => Role.getRoleByName(el)).toList()
          : <Role>[];
    }

    if (json['bank_cards'] != null) {
      final bankCardsJson = List.from(json['bank_cards']);
      bankCards = bankCardsJson.isNotEmpty
          ? bankCardsJson.map<BankCard>((el) => BankCard.fromJson(el)).toList()
          : <BankCard>[];
    }

    final kyc = json['kyc_validations'];

    return User(
      id: '${json['id']}',
      createdAt: json['created'],
      updatedAt: json['updated'],
      username: json['username'],
      phone: json['phone'].toString(),
      prefix: json['prefix'].toString(),
      hasPin: json['has_pin'],
      email: json['email'],
      kycId: kyc['id'],
      zip: kyc['zip'],
      birthDate: kyc['date_birth'],
      gender: kyc['gender'],
      privateTosLtab: json['private_tos_campaign'],
      privateTosCulture: json['private_tos_campaign_culture'],
      hasSavedCards: json['has_saved_cards'] ?? false,
      isKycManager: json['is_kyc_manager'] ?? false,
      selectedAccount: Account.fromJson(json['active_group']),
      accounts: List.from(json['activeAccounts']).map((el) => Account.fromJson(el)).toList(),
      roles: roles,
      bankCards: bankCards,
    );
  }
}
